import os
import pandas as pd
import numpy as np
from tkinter import filedialog
from tkinter import Tk
import re
import matplotlib.pyplot as plt

# Initialize Tkinter
root = Tk()
root.withdraw()

# Ask the user to select multiple Excel files
file_paths = filedialog.askopenfilenames(title='Select Excel files', filetypes=[('Excel files', '*.csv *.xls')])

# Dictionary to store the epsilon r values for each file
epsilon_r_dict = {}

dfs = []
file = 0
# Loop over the selected files
for file_path in file_paths:
    match = re.search(r'(\d+)um', file_path)
    if match:
        d_str = match.group(1)
        d = float(d_str) / 1000000  # Convert from micrometers to millimeters
    else:
        raise ValueError('File name does not contain d value in micrometers')
    # Read the Excel file into a pandas DataFrame
    df = pd.read_csv(file_path, skiprows=11)

    # Calculate the epsilon r values
    A = 19.83 * 19.8 * (1e-3)**2  # Area in m^2
    # d = 0.42 * 1e-3  # Distance between plates in m
    df = df[df['Cs[F]'] != np.inf].dropna(subset=['Cs[F]'])
    C = df['Cs[F]']  # Capacitance in Farads
    epsilon_0 = 8.8541878128e-12  # Permittivity of free space
    epsilon_r = (C * (d / A)) / epsilon_0  # Relative permittivity
    df = df[df['D'] != np.inf].dropna(subset=['D'])
    delta_loses = df['D']
    # Store the epsilon r values in the dictionary
    file_name = re.search(r'(\d+)um', file_path).group(0) + "_iteration_{}"
    file_name = file_name.format(file+1)
    # data.append([file_name, epsilon_r.values.tolist(), delta_loses.values.tolist()])
    # # epsilon_r_dict[file_name] = [epsilon_r.values.tolist(),delta_loses.values.tolist()]
    # add freq column
    freq = df['Frequency[Hz]']
    # file += 1
    # Store the epsilon r values in a new DataFrame
    iteration_df = pd.DataFrame({'Frequency[Hz]':freq,'epsilon_r': epsilon_r, 'tan(delta)': delta_loses})
    iteration_df = iteration_df.replace('', np.nan)
    iteration_df = iteration_df.dropna(subset=['epsilon_r', 'tan(delta)'])
    # Add a column for the iteration number
    iteration_df[file_name] = file + 1
    # Append the DataFrame to the list
    dfs.append(iteration_df)

    file += 1


# Create a DataFrame from the epsilon r dictionary
# df_epsilon_r = pd.DataFrame(data, columns=['filename', 'epsilon_r', 'delta_losses'])
df_epsilon_r = pd.concat(dfs, axis=1)
df_epsilon_r = df_epsilon_r.dropna()


### ploting
ax = df_epsilon_r.plot.scatter(x=df_epsilon_r.filter(like='Frequency[Hz]').columns, y=df_epsilon_r.filter(like='epsilon_r').columns)
colors = plt.cm.rainbow(np.linspace(0, 1, len(df_epsilon_r.columns)))
for i, c in enumerate(ax.collections):
    c.set_color(colors[i])
# Axis labels
ax.set_xlabel('Frequency [Hz]')
ax.set_ylabel('Epsilon')
ax.set_title('Scatter Plot of Epsilon vs Frequency')

# Display the plot
plt.show()
# Save the DataFrame to a CSV file
output_csv_path = os.path.join(os.path.dirname(__file__), 'output.csv')
df_epsilon_r.to_csv(output_csv_path, index=False)
